/**
 *  Namespace defines additional handlebars helpers. That could be used in whole storefront
 *
 *  <ul>
 *  <li>
 *      ifPageDisplay - render block or not depend on current ccpage
 *      <pre>
 *              {{#ifPageDisplay 'ccrz__HomePage'}}
 *              <h1>Home page</h1>
 *              {{/ifPageDisplay}}
 *      </pre>
 *  </li>
 *  <li>
 *      ifConfigValue - helpers renders block or not depend on current storefront and sales organization
 *      <pre>
 *              {{#ifConfigValue 'Checkout_DeliveryDate__c'}}
 *                  <div class="col-md-5 col-xs-12  section">
 *                      <div><label>{{pageLabelMap 'CHECKOUT_DELIVERYDATE_LABEL'}} </label></div>
 *                       <div><input type="date" name="date" /></div>
 *                  </div>
 *              {{/ifConfigValue}}
 *      </pre>
 * </li>
 *  </ul>
 */
d('helpers',['hbs', 'util'], function(Handlebars, util) {
    var initialized = false;
    return {
        registerHelpers : function(){
            if(!initialized){
                // show elements on pageName
                Handlebars.registerHelper('ifPageDisplay', function(pageName, options) {
                    if (CCRZ.pagevars.currentPageName.toLowerCase() ==  pageName.toLowerCase()) {
                        return options.fn(this);
                    }
                    else {
                        return options.inverse(this);
                    }
                });
                // show elements on pageName
                Handlebars.registerHelper('ifNotPageDisplay', function(pageName, options) {
                    return (CCRZ.pagevars.currentPageName.toLowerCase() != pageName.toLowerCase()) ? true : false;
                });
                //get value from config
                Handlebars.registerHelper('getConfig', function(str) {
                    return CCRZ.pagevars.pageConfig[str];
                });
                //check if obejct is empty
                Handlebars.registerHelper('ifObjectExistButIsEmpty', function(obj) {
                   if(obj.length == 1){
                       var isEmpty = _.isEmpty(obj[0]);
                       if (isEmpty) { return true; }
                   }
                   else if (obj.length == 0){return true; }
                   else{ return false; }
                });
                //return object size
                Handlebars.registerHelper("objectSize", function(object)
                {
                    if(object){
                        return Object.keys(object).length;
                    }
                    return null;
                });
                Handlebars.registerHelper('ifOneItemWithOneQty', function(productArray, groupModel, modelOrder){
                    var size = productArray.length;
                    var flag = false;
                    if(modelOrder){
                        var itemList = groupModel[0].attributes.EOrderItemsS[0];
                    }else{
                        var itemList = groupModel[0].attributes.ECartItemsS[0];
                    }
                    if(size == 1 && itemList){
                        var qty = itemList.quantity;
                        if(qty ==1){ flag = true;}
                    }
                    return flag;
                });
                Handlebars.registerHelper('or', function( val1, val2 ){
                    if ( val1 || val2 ) {
                        return true;
                    } else {
                        return false;
                    }
                });
                Handlebars.registerHelper('and', function( val1, val2 ){
                    if ( val1 && val2 ) {
                        return true;
                    } else {
                        return false;
                    }
                });
                Handlebars.registerHelper('logicalAND', function( val1, val2, options){
                    if ( val1 && val2 ) {
                        return options.fn(this);
                    }
                    return options.inverse(this);
                });
                Handlebars.registerHelper('logicalOR', function( val1, val2, options){
                    if ( val1 || val2 ) {
                        return options.fn(this);
                    }
                    return options.inverse(this);
                });
                Handlebars.registerHelper('subcategoryIsCurrentCategory', function(subcategories, currentCategoryId){
                    var isCurrent = false;
                    _.each(subcategories, function(subcategory) {
                        var subId = subcategory.categoryID;
                        if (subId === currentCategoryId) {
                            isCurrent = true;
                        }
                    });
                    return isCurrent;
                });
                Handlebars.registerHelper('formatCurrencyAdd', function(amount, addAmount){
                    let total = amount + addAmount;
                    let numFormat = new Intl.NumberFormat(CCRZ.pagevars.remoteContext.userLocale.replace('_','-'), { 
                        style: 'currency', 
                        currency: CCRZ.pagevars.remoteContext.userIsoCode, 
                        minimumFractionDigits: 2, 
                    }); 
                    return numFormat.format(total);
                });
                Handlebars.registerHelper('formatCurrencySub', function(amount, addAmount){
                    let total = amount - addAmount;
                    let numFormat = new Intl.NumberFormat(CCRZ.pagevars.remoteContext.userLocale.replace('_','-'), { 
                        style: 'currency', 
                        currency: CCRZ.pagevars.remoteContext.userIsoCode, 
                        minimumFractionDigits: 2, 
                    }); 
                    return numFormat.format(total);
                });
                Handlebars.registerHelper('formatCurrencyDecimal', function(amount){
                    let numFormat = new Intl.NumberFormat(CCRZ.pagevars.remoteContext.userLocale.replace('_','-'), { 
                        style: 'currency', 
                        currency: CCRZ.pagevars.remoteContext.userIsoCode, 
                        minimumFractionDigits: ((amount % 1 == 0)?2:2), 
                    }); 
                    return numFormat.format(amount);
                });
                Handlebars.registerHelper('formatCurrencyMul', function(amount, quantity){
                    let total = amount * quantity;
                    let numFormat = new Intl.NumberFormat(CCRZ.pagevars.remoteContext.userLocale.replace('_','-'), { 
                        style: 'currency', 
                        currency: CCRZ.pagevars.remoteContext.userIsoCode, 
                        minimumFractionDigits: 2, 
                    }); 
                    return numFormat.format(total);
                });
                Handlebars.registerHelper('formatCurrency', function(amount){
                let numFormat = new Intl.NumberFormat(CCRZ.pagevars.remoteContext.userLocale.replace('_','-'), { 
                    style: 'currency', 
                    currency: CCRZ.pagevars.remoteContext.userIsoCode, 
                    minimumFractionDigits: 2, 
                }); 
                return numFormat.format(amount);
                });
                Handlebars.registerHelper('formatCurrencyISO', function(amount, userIsoCode){
                    let numFormat = new Intl.NumberFormat(CCRZ.pagevars.remoteContext.userLocale.replace('_','-'), { 
                        style: 'currency', 
                        currency: userIsoCode, 
                        minimumFractionDigits: 2, 
                    }); 
                return numFormat.format(amount);
                });
                Handlebars.registerHelper('formatDateOrder', function(date){
                    let options ={day: 'numeric', month: 'short', year: 'numeric'};
                    let conDate = new Date(Number(date));
                    let dateFormat = new Intl.DateTimeFormat(CCRZ.pagevars.remoteContext.userLocale.replace('_','-'), options).format(conDate);
                    return dateFormat.split('-').join('/');
                });
                Handlebars.registerHelper('formatDate', function(date){
                    if(typeof date != 'undefined'){
                        let options ={day: 'numeric', month: 'numeric', year: 'numeric'};
                        let conDate = new Date(Number(date)), locale='en-US';
                        conDate = new Date(conDate.toLocaleString( locale, { timeZone: "UTC" }));
                        let dateFormat = new Intl.DateTimeFormat(CCRZ.pagevars.remoteContext.userLocale.replace('_','-'), options).format(conDate);
                        return dateFormat.split('-').join('/');
                    } else {
                        return '';
                    }
                });
                Handlebars.registerHelper('checkRenewal', function(renDate, status, autoRenew){
                    if(autoRenew){
                        if(status == 'Inactive-Expired'){
                            return 'btn btn-info renewSubsc';
                        } else {
                            return 'btn btn-info renewSubsc disabled';
                        }
                    } else {
                        let renewalDate = new Date(renDate), currDate = new Date(), renewalStart = renewalDate;
                        renewalStart = renewalStart.setMonth(renewalStart.getMonth() - 1);
                        renewalDate.setFullYear(renewalDate.getFullYear() + 1);
                        if((currDate <= renewalDate && status == 'Inactive-Expired') || (status == 'Active' && currDate >= renewalStart)){
                            return 'btn btn-info renewSubsc';
                        } else {
                            return 'btn btn-info renewSubsc disabled';
                        }
                    }
                });
                Handlebars.registerHelper('getImageURL', function(imagePath){
                    return CCRZ.pagevars.themeBaseURL + imagePath;
                });
                Handlebars.registerHelper('getSubTotal', function(subTotal, hasCoupon, items, type){
                    if(hasCoupon){
                        if(items && items.length > 0){
                            for(let i = 0; i < items.length; i++){
                                switch(type) {
                                    case 'cart':
                                        if(items[i].attributes.productType == 'Coupon'){
                                            subTotal += -(items[i].attributes.itemTotal);
                                        } else {
                                            subTotal += (items[i].attributes.baseDiscount?items[i].attributes.baseDiscount:0);
                                        }
                                        break;
                                    case 'review':
                                        if(items[i].productType == 'Coupon'){
                                            subTotal += -(items[i].itemTotal);
                                        } else {
                                            subTotal += (items[i].baseDiscount?items[i].baseDiscount:0);
                                        }
                                        break;
                                    case 'order':
                                        if(items[i].productType == 'Coupon'){
                                            subTotal += -(items[i].itemTotal);
                                        } else {
                                            subTotal += (items[i].absoluteDiscount?items[i].absoluteDiscount:0);
                                        }
                                        break;
                                    default:
                                        return '';
                                }
                            }
                        }
                        
                    }
                    return subTotal;
                }); 
                Handlebars.registerHelper('getDiscountPriceOrder', function(cartData){
                    let discountAmt = 0;
                    if(cartData.hasCoupon){
                        if(cartData.orderItems && cartData.orderItems.length > 0){
                            for(let i = 0; i < cartData.orderItems.length; i++){
                                if(cartData.orderItems[i].productType == 'Product'){
                                    discountAmt += (cartData.orderItems[i].absoluteDiscount?cartData.orderItems[i].absoluteDiscount:0);
                                } else if(cartData.orderItems[i].productType == 'Coupon'){
                                    discountAmt += -(cartData.orderItems[i].itemTotal);
                                }
                            }
                        }
                        
                    }
                    let numFormat = new Intl.NumberFormat(CCRZ.pagevars.remoteContext.userLocale.replace('_','-'), { 
                        style: 'currency', 
                        currency: cartData.currencyCode, 
                        minimumFractionDigits: 2, 
                    }); 
                    return numFormat.format(discountAmt);
                });
                Handlebars.registerHelper('getDiscountPriceCheckout', function(cartData){
                    let discountAmt = 0;
                    if(cartData.hasCoupon){
                        if(cartData.cartItems && cartData.cartItems.length > 0){
                            for(let i = 0; i < cartData.cartItems.length; i++){
                                if(cartData.cartItems[i].productType == 'Product'){
                                    discountAmt += (cartData.cartItems[i].absoluteDiscount?cartData.cartItems[i].baseDiscount:0);
                                } else if(cartData.cartItems[i].productType == 'Coupon'){
                                    discountAmt += -(cartData.cartItems[i].itemTotal);
                                }
                            }
                        }
                    }
                    let numFormat = new Intl.NumberFormat(CCRZ.pagevars.remoteContext.userLocale.replace('_','-'), { 
                        style: 'currency', 
                        currency: cartData.currencyISOCode, 
                        minimumFractionDigits: 2, 
                    }); 
                    return numFormat.format(discountAmt);
                });
                Handlebars.registerHelper('getDiscountPrice', function(cartData){
                    let discountAmt = 0;
                    if(cartData.hasCoupon){
                        if(cartData.ECartItemsS && cartData.ECartItemsS.models && cartData.ECartItemsS.models.length > 0){
                            for(let i = 0; i < cartData.ECartItemsS.models.length; i++){
                                if(cartData.ECartItemsS.models[i].attributes.productType == 'Product'){
                                    discountAmt += (cartData.ECartItemsS.models[i].attributes.absoluteDiscount?cartData.ECartItemsS.models[i].attributes.absoluteDiscount:0);
                                } else if(cartData.ECartItemsS.models[i].attributes.productType == 'Coupon'){
                                    discountAmt += -(cartData.ECartItemsS.models[i].attributes.itemTotal);
                                }
                            }
                        }
                        
                    }
                    let numFormat = new Intl.NumberFormat(CCRZ.pagevars.remoteContext.userLocale.replace('_','-'), { 
                        style: 'currency', 
                        currency: cartData.currencyISOCode, 
                        minimumFractionDigits: 2, 
                    }); 
                    return numFormat.format(discountAmt);
                });
                Handlebars.registerHelper('checkProdDisability', function(tierList, subQuantity, options){
                    if(tierList.length > 0){
                        for(let i = 0; i < tierList.length; i++){
                            if(tierList[i].currencyCode == CCRZ.pagevars.remoteContext.userIsoCode && (tierList[i].minQuantity == tierList[i].maxQuantity || subQuantity == tierList[i].maxQuantity)){
                                return options.fn(this);
                            }
                        }
                        return options.inverse(this);
                    }
                    return options.inverse(this);
                });
                Handlebars.registerHelper('checkActionDisability', function(status, isAddLicenses, autoRenew, renDate){
                    let renewalDate = new Date(renDate), currDate = new Date();
                    if(isAddLicenses == 'true'){
                        if(status == 'Active'){
                            if(!autoRenew){
                                if(currDate < renewalDate){
                                    return 'btn btn-danger licenceModal disabled';
                                } else {
                                    return 'btn btn-danger licenceModal';
                                }
                            } else {
                                return 'btn btn-danger licenceModal';
                            }
                        } else {
                            return 'btn btn-danger licenceModal disabled';
                        }
                    } else {
                        if(status == 'Active'){
                            if(!autoRenew){
                                if(currDate < renewalDate){
                                    return 'btn btn-default solidFocus addBoltOn width100 disabled';
                                } else {
                                    return 'btn btn-default solidFocus width100 addBoltOn';
                                }
                            } else {
                                return 'btn btn-default solidFocus width100 addBoltOn';
                            }
                        } else {
                            return 'btn btn-default solidFocus width100 addBoltOn disabled';
                        }
                    }
                });
                Handlebars.registerHelper('checkSubActivity', function(status){
                    if(status == 'Active'){
                        return 'subscribed-item card';
                    } else {
                        return 'subscribed-item card inactive';
                    }
                });
                Handlebars.registerHelper('calculateTotalPDP', function(productPrice, addOnPrice, quantity){
                    return util.formatCurrency((productPrice * quantity) + (addOnPrice * quantity));
                });
                Handlebars.registerHelper('checkDefault', function(countryCode){
                    return (countryCode == CCRZ.pagevars.pageLabels['Egress_checkout_defaultCountryCode']?'selected':'');
                });
                Handlebars.registerHelper('getCurrentUser', function(){
                    return CCRZ.currentUser.FirstName + ' ' + CCRZ.currentUser.LastName;
                });
                Handlebars.registerHelper('ifUserAvailable', function(options){
                    return (typeof CCRZ.currentUser != 'undefined') ? options.fn(this) : options.inverse(this);
                });
                Handlebars.registerHelper('egressThemeUrl', function(){
                    return CCRZ.pagevars.themeBaseURL;
                });
                Handlebars.registerHelper('getCurrentPLPCategoryId', function(){
                    if(CCRZ.productListPageModel && CCRZ.productListPageModel.attributes.isCategory){
                        var categoryId = CCRZ.productListPageModel.attributes.category.categoryID;  
                        return categoryId;
                    } else if(CCRZ.productListPageModel && CCRZ.productListPageModel.attributes.isSearch){
                        return '';
                    }
                });
                Handlebars.registerHelper('isExpired', function(_this, options) {
                    if (new Date(_this.installmentLastDate) <= new Date()) {
                        return options.fn(this);
                    }
                    return options.inverse(this);
                });
                Handlebars.registerHelper('isLoggedInMenu', function(isLoggedInMenu, options) {
                    if (isLoggedInMenu && isLoggedInMenu == CCRZ.pagevars.isGuest) {
                        return options.fn(this);
                    }
                    return options.inverse(this);
                });
                Handlebars.registerHelper('checkCartQuantity', function(cartLength, options) {
                    if (cartLength > 1) {
                        return options.fn(this);
                    }
                    return options.inverse(this);
                });
                Handlebars.registerHelper('recurringGreater', function(monthlyPrice, yearlyPrice, options) {
                    if (monthlyPrice > 0 || yearlyPrice > 0) {
                        return options.fn(this);
                    }
                    return options.inverse(this);
                });
                Handlebars.registerHelper('switch', function(value, options) {
                    this.switch_value = value;
                    return options.fn(this);
                });
                
                Handlebars.registerHelper('case', function(value, options) {
                    if (value == this.switch_value) {
                        return options.fn(this);
                    }
                });
                Handlebars.registerHelper('ifCond', function (v1, operator, v2, options) {
                    switch (operator) {
                        case '==':
                            return (v1 == v2) ? options.fn(this) : options.inverse(this);
                        case '===':
                            return (v1 === v2) ? options.fn(this) : options.inverse(this);
                        case '!=':
                            return (v1 != v2) ? options.fn(this) : options.inverse(this);
                        case '!==':
                            return (v1 !== v2) ? options.fn(this) : options.inverse(this);
                        case '<':
                            return (v1 < v2) ? options.fn(this) : options.inverse(this);
                        case '<=':
                            return (v1 <= v2) ? options.fn(this) : options.inverse(this);
                        case '>':
                            return (v1 > v2) ? options.fn(this) : options.inverse(this);
                        case '>=':
                            return (v1 >= v2) ? options.fn(this) : options.inverse(this);
                        case '&&':
                            return (v1 && v2) ? options.fn(this) : options.inverse(this);
                        case '||':
                            return (v1 || v2) ? options.fn(this) : options.inverse(this);
                        default:
                            return options.inverse(this);
                    }
                });
              
                Handlebars.registerHelper('html_decoder', function(encodedStr) {
                  if(encodedStr && encodedStr.length > 0){
                    var parser = new DOMParser;
                    var dom = parser.parseFromString(
                        '<!doctype html><body>' + encodedStr,
                        'text/html');
                    var decodedString = dom.body.textContent;
                    if(decodedString){
                      return decodedString; 
                    }else{
                      return ''; 
                    }
                  }
                  return '';
                });
              
                Handlebars.registerHelper('currencyCodeHelper', function (currnecyISo) {
                  let currencySymbol;
                  if(currnecyISo == 'USD'){
                      currencySymbol = '$';
                  }else if(currnecyISo == 'GBP'){
                      currencySymbol = '£';
                  } else if(currnecyISo == 'EUR'){
                    currencySymbol = '€';
                }  
                  return currencySymbol;
              
                });
                Handlebars.registerHelper('getLocaleName', function(locale){
                    return CCRZ.pagevars.pageLabels['Country_Name_' + locale];
                 });
                Handlebars.registerHelper('getContactName', function(){
                return _.isUndefined(CCRZ.pagevars.contactInfo)?'<i class="fa fa-spinner fa-spin">':CCRZ.pagevars.contactInfo.Name;
                });
                Handlebars.registerHelper('getCurrentIterationEan', function(productId, productList){
                   var item = productList[productId];
                   return item.cccustEAN;
                });
                Handlebars.registerHelper('findPaginationIdTarget', function(){
                    var target = $("[id$='pagin-target']");
                    var targetId = target.attr("id");
                    return targetId;
                });
                Handlebars.registerHelper('getNumberOfAlerts', function(){
                    if ( CCRZ.AlertView && CCRZ.AlertView.model ) {
                         return CCRZ.AlertView.model.length;
                     } else {
                         return 0;
                     }
                });
                Handlebars.registerHelper('returnListOfAttributes', function(productList, nameOfAttribute){
                    var attributeArray = [];
                    _.each(productList, function(product) {
                        attributeArray.push(product[nameOfAttribute]);
                    });
                    return attributeArray.join(', ');
                });
                Handlebars.registerHelper('getAlertsTotal', function(){
                    if ( CCRZ.AlertView && CCRZ.AlertView.model ) {
                        var cookie = CCRZ.getCookie('seenAlerts');
                        if (!cookie) {
                            cookie = [];
                            var jsonidList = JSON.stringify(cookie);
                            CCRZ.setCookieWithPath('seenAlerts', jsonidList, 30, '/');
                            var cookie = CCRZ.getCookie('seenAlerts');
                            var seen = JSON.parse(cookie);
                        } else {
                            var seen = JSON.parse(cookie);
                        }

                        var seenAmount = seen.length,
                            alerts = CCRZ.AlertView.model.length;

                        if ( seen && seenAmount > 0 ) {
                            return alerts - seenAmount;
                        } else {
                            return alerts;
                        }
                    } else {
                        return 0;
                    }
                });
                Handlebars.registerHelper('equals', function(value1, value2){
                    if (value1 === value2) {
                        return true;
                    } else {
                        return false;
                    }
                });
                Handlebars.registerHelper('isBundle', function(productType, options) {
                    if (productType == 'Dynamic Kit' || productType == 'Kit' || productType == 'Bundle') {
                        return options.fn(this);
                    }
                    return options.inverse(this);
                });
                Handlebars.registerHelper('ifGreaterDate', function(startDate, options) {
                    let currentDate = new Date();
                    startDate = new Date(startDate);
                    if (startDate > currentDate) {
                        return options.fn(this);
                    }
                    return options.inverse(this);
                });
                Handlebars.registerHelper('ifGreaterOrEqual', function(v1, v2, options) {
                    if (v1 >= v2) {
                        return options.fn(this);
                    }
                    return options.inverse(this);
                });
                Handlebars.registerHelper('reverse', function (arr) {
                    arr.reverse();
                });
                Handlebars.registerHelper('debug2',function(data,desc){
                    console.log(data);
                    return '';
                });
                Handlebars.registerHelper('getProductType',function(){
                    return CCRZ.detailData.jsonProductData.product.prodBean.ProductType;
                });
                Handlebars.registerHelper('getResourceUrl', function(str) {
                     return CCRZ.pagevars.themeBaseURL + str;
                });
                Handlebars.registerHelper('ifPageKey', function(pageKey){
                   return (pageKey == CCRZ.getQueryParam('pageKey')) ? true : false;
                });
                Handlebars.registerHelper('getShippingProvider',function(index){
                    if ( index >= 0 && CCRZ.LLIShippingDetailModel ) {
                        var model = CCRZ.LLIShippingDetailModel.attributes.cartData.ECartItemGroupsS.models[index];
                        var shippingProvider = model.attributes.shipProvider;
                        return shippingProvider;
                    } else {
                        return Handlebars.helpers.pageLabelMap('ShippingRate_Provider_Unknown');
                    }
                });
                Handlebars.registerHelper('getShippingProviderByMethod',function(shipGroup){
                    if ( shipGroup && CCRZ.LLIShippingDetailModel ) {
                        var shippingOptions = shipGroup.shippingOptions;
                        var provider = Handlebars.helpers.pageLabelMap('ShippingRate_Provider_Unknown');
                        _.each(shippingOptions, function(option) {
                            if (option.serviceName == shipGroup.shipMethod) {
                                provider = option.provider;
                            }
                        }, shipGroup);
                        return provider;
                    } else {
                        return Handlebars.helpers.pageLabelMap('ShippingRate_Provider_Unknown');
                    }
                });
                Handlebars.registerHelper('checkIfInCartIsCoupon', function(cartItems){
                   var checkResult = false;

                   $.each(cartItems.models, function(index, element) {
                        if(element.attributes.cartItemType == 'Coupon'){
                           checkResult = true;
                        }
                   });

                   return checkResult;
                });

                Handlebars.registerHelper('allDiscount', function(cartItems){
                   var discount = 0;

                   if(Handlebars.helpers.checkIfInCartIsCoupon(cartItems)){
                       $.each(cartItems.models, function(index, element) {
                           if(element.attributes.cartItemType == 'Coupon'){
                              discount += -(element.attributes.SubAmount);
                           }
                      });
                   }else{
                        $.each(cartItems.models, function(index, element) {
                            if (element.attributes.absoluteDiscount) {
                                discount += element.attributes.absoluteDiscount;
                            }
                        });
                   }

                   return discount;
                });
                Handlebars.registerHelper('checkIfShowSaveLabels', function(cartItems){
                    var checkResult = false;
                    var checkVariable = Handlebars.helpers.allDiscount(cartItems);
                    if(checkVariable != 0){
                        checkResult = true;
                    }

                    return checkResult;
                });
                //check is mobile or width less than breakpoint
                Handlebars.registerHelper('checkMobile', function(breakpoint){
                    var screensize = $(window).width(),
                    breakpoint = Number(breakpoint);

                    return (screensize < breakpoint) ? true : false;
                });
                Handlebars.registerHelper('getPrintOrder',function(orderEncId){
                    return CCRZ.pagevars.currSiteURL + 'hbegress_print_orderConfirmation?id=' + orderEncId;
                });
                Handlebars.registerHelper('convertDate',function(dataToConvert){
                    var format = CCRZ.pagevars.pageLabels['Date_Format'];
                    return util.formatDate(new Date(dataToConvert),format);
                });
                Handlebars.registerHelper('compareDate',function(dateToCompare){
                    if(dateToCompare != undefined && dateToCompare != ''){
                        var todayDate = new Date(),
                            convertDate = new Date(dateToCompare);
                        return (todayDate.toLocaleDateString() < convertDate.toLocaleDateString()) ? true : false;
                    }else{
                        return false;
                    }
                });
                Handlebars.registerHelper('ifNotPastDate',function(dateToCompare){
                     if(dateToCompare != undefined && dateToCompare != ''){
                        var todayDate = new Date(),
                            convertDate = new Date(dateToCompare);
                        return (Date.parse(convertDate.toLocaleDateString()) >= Date.parse(todayDate.toLocaleDateString())) ? true : false;
                     }else{
                        return true;
                     }
                });
                Handlebars.registerHelper('restrictColumnFile', function(value, productFamily,options){
                    if ((productFamily == 'Columns and Features' && (value == 'TIF' || value == 'JPG'))
                        || (productFamily == 'Comics' && value == 'TXT') || (productFamily == 'Puzzles and Games' && value == 'TXT')) {
                        return options.inverse(this);
                    } else {

                    }
                    return options.fn(this);
                });
                Handlebars.registerHelper('isFilter', function(sptType, productFamily,options){
                    if (sptType == 'Evergreen' && (productFamily == 'Comics' || productFamily == 'Political Cartoons')) {
                        return options.fn(this);
                    } else {
                        return options.inverse(this);
                    }
                });
                Handlebars.registerHelper('showFileName', function(productFamily, options){
                    if ((productFamily == 'Comics' || productFamily == 'Political Cartoons' || productFamily == 'Puzzles and Games')) {
                        return options.inverse(this);
                    } else {
                        return options.fn(this);
                    }
                });
                Handlebars.registerHelper('isLoggedUser', function(options) {
                    if (CCRZ.pagevars.isGuest) {
                        return options.inverse(this);
                    } else {
                        return options.fn(this);
                    }
                });
                Handlebars.registerHelper('ifEven', function(conditional, options) {
                  if((conditional % 2) == 0) {
                    return options.fn(this);
                  } else {
                    return options.inverse(this);
                  }
                });
                Handlebars.registerHelper('strong', function(value){
                    value = value.replace(new RegExp('<em>', 'g'), '<strong>');
                    value = value.replace(new RegExp('</em>', 'g'), '</strong>');
                    return value;
                });
                Handlebars.registerHelper('productLinkSearch', function(product, styleClass, options) {
                    var SKU = '';
                    if (!_.isUndefined(product)) {
                        if (product.linkURL) {
                            SKU = product.linkURL;
                        } else if (product.sku) {
                            SKU = product.sku;
                        } else if (product.SKU) {
                            SKU = product.SKU;
                        } else if (product.productSKU) {
                            SKU = product.productSKU;
                        }
                        var productName = product.highlightName || product.name
                        productName = Handlebars.helpers.strong(productName);
                        var linkObj = {
                            'name': productName,
                            'SKU': SKU,
                            'friendlyUrl': product.friendlyUrl,
                            'openInNewWindow': product.openInNewWindow
                        };
                        var productJSON = _.escape(JSON.stringify(linkObj));
                    }
                    var content = '';
                    if (productName) {
                        content = _.unescape(productName);
                    }
                    if (options && options.hash['image'])
                        content = options.hash['image'];
                    if (options && options.hash['text']) {
                        content = _.escape(_.unescape(options.hash['text']));
                    }
                    return new Handlebars.SafeString("<a href=\"javascript:void(0);\" onClick=\"CCRZ.openPDP(this)\" class='" + styleClass + " gp_prod' data-product= '" + productJSON + "' data-id= '" + SKU + "'>" + content + "</a>")
                });
                Handlebars.registerHelper('missSpaces', function(string) {
                    if (string)
                        return string.replace(' ','');
                });
                Handlebars.registerHelper('sortCategories', function(categories, field) {
                    var result = _.sortBy(categories, function(e){
                        if (e.children){
                            e.children = _.sortBy(e.children, function(f){
                                return f[field];
                            });
                        }
                        return e[field];
                    });
                    return result;
                });
                //index offset helper
                Handlebars.registerHelper('offset', function(number, options) {
                    if(typeof(number) === 'undefined' || number === null)
                        return null;
                    return number + (options.hash.offset || 0);
                });
                //set image from static resources
                Handlebars.registerHelper('setImgSrc', function(str) {
                    var currSiteURL = CCRZ.pagevars.currSiteURL,
                        themeBaseURL = "",
                        themeBaseURLArray = CCRZ.pagevars.themeBaseURL.split("/");
                    for(var i = 0; i < themeBaseURLArray.length ; i++){
                        if(themeBaseURLArray[i] && themeBaseURLArray[i].toLowerCase() != CCRZ.pagevars.storefrontName.toLowerCase()){
                            themeBaseURL+=themeBaseURLArray[i]+"/";
                        }
                    }
                    return currSiteURL + themeBaseURL + str;
                });
                //overriden 'displayImage' helper
                Handlebars.registerHelper('displayImage', function(obj, styleClass, options) {
                    var blankImage = Handlebars.helpers.getReplacementImage();
                    var onerrorSrc = Handlebars.helpers.onerrorSrc();
                    if (options && options.hash['src']){
                        var imgSrc = _.escape(options.hash['src']);
                    }else{
                        var imgSrc = CCRZ.processImageURL(obj, styleClass, options);
                    }
                    if (imgSrc === ''){
                        imgSrc = blankImage;
                    }
                    var alt = '';
                    if (options && options.hash['alt']){
                        alt = _.escape(options.hash['alt']);
                    }
                    var dataId = "";
                    if (options && options.hash['dataId']){
                        dataId = _.escape(options.hash['dataId']);
                    }
                    if (imgSrc.length > 0){
                        return new Handlebars.SafeString("<img class='" + styleClass +"' src='" + imgSrc + "' alt='" + alt + "' data-id='" + dataId + "' onerror='" + onerrorSrc + "'/>");
                    }else{
                        return new Handlebars.SafeString("<img class='" + styleClass + " " + CCRZ.pagevars.userLocale + " noImg' alt='" + alt + "' data-id='" + dataId + "' onerror='" + onerrorSrc + "'/>");
                    }
                });
                
                //return current date in a format
                Handlebars.registerHelper('getClassByCount', function(cartCount){
                    if(CCRZ.pagevars.isGuest){
                        return 'twoDigit';
                    } else {
                        cartCount = cartCount + '';
                        return (cartCount.length <= 2?'twoDigit':'threeDigit');
                    }
                });
                //return current date in a format
                Handlebars.registerHelper('getCurrentDate', function(){
                    return util.formatDate(new Date(), 'dd/mm/yyyy');
                });
                //return product image uri link
                Handlebars.registerHelper('getImageURI', function(media, page){
                    return media[0].URI;
                });
                //return default product image src link
                Handlebars.registerHelper('getReplacementImage', function(){
                    var replacementImg = Handlebars.helpers.setImgSrc('images/image-not-available.png');
                    return replacementImg;
                });
                //return onerror attr with replacement image
                Handlebars.registerHelper('onerrorSrc', function(){
                    return 'this.onerror=null;this.src="' + Handlebars.helpers.getReplacementImage() + '"';
                });
                Handlebars.registerHelper('getSiteURL', function(){
                    return CCRZ.pagevars.currSiteURL;
                });
                Handlebars.registerHelper('myAccountByViewState', function(viewState) {
                    return new Handlebars.SafeString(CCRZ.pagevars.currSiteURL + 'ccrz__MyAccount?' + 'viewState=' + viewState + '&cartID=' + CCRZ.pagevars.currentCartID  + getCSRQueryString());
                });
                //return link to Product Detail Page from SKU
                Handlebars.registerHelper('PDPLink', function(sku) {
                    return new Handlebars.SafeString(CCRZ.pagevars.currSiteURL + 'ccrz__ProductDetails?' + 'sku=' + sku + '&cartID=' + CCRZ.pagevars.currentCartID  + getCSRQueryString());
                });
                //return link to Product Detail Page for externalParams
                Handlebars.registerHelper('PDPLinkCustom', function(_this) {
                    let link = new Handlebars.SafeString(CCRZ.pagevars.currSiteURL + 'ccrz__ProductDetails?' + 'sku=' + _this.productR.SKU + '&cartID=' + CCRZ.pagevars.currentCartID  + getCSRQueryString());
                    if((_this.subProdTermR && _this.subProdTermR.KFB2BSptType == 'Reprint') || _this.KFB2BSptType == 'Reprint'){
                        link += '&isReprint=' + true + '&reprintId=' + _this.extSKU;
                    } else {
                        link += '&isReprint=' + false;
                    }
                    return link;
                });
                //return link to Product Detail Page for Related Products
                Handlebars.registerHelper('PDPLinkRelated', function(sku) {
                    let link = new Handlebars.SafeString(CCRZ.pagevars.currSiteURL + 'ccrz__ProductDetails?' + 'sku=' + sku + '&cartID=' + CCRZ.pagevars.currentCartID  + getCSRQueryString());
                    link += '&isReprint=' + false;
                    return link;
                });
                Handlebars.registerHelper('goToPDP', function(product) {
                    let sku = product.SKU, isReprint = (typeof product.isReprint != 'undefined'?product.isReprint:false);
                    return new Handlebars.SafeString(CCRZ.pagevars.currSiteURL + 'ccrz__ProductDetails?' + 'sku=' + sku  + '&isReprint=' + isReprint + (isReprint?('&reprintId=' + product.sfid):'') + '&cartID=' + CCRZ.pagevars.currentCartID  + getCSRQueryString());
                });
                Handlebars.registerHelper('goToHome', function() {
                    return '/' + CCRZ.pagevars.storefrontName;
                });
                Handlebars.registerHelper('goToLogin', function() {
                    let startURL = CCRZ.pagevars.sitePrefix + '/' + CCRZ.pagevars.currentPageName + '?';
                    for(let key in CCRZ.pagevars.queryParams){
                        startURL += key + '=' + CCRZ.pagevars.queryParams[key] + '&';
                    }
                    startURL = startURL.slice(0,-1);
                    return new Handlebars.SafeString(CCRZ.pagevars.currSiteURL + 'ccrz__CCSiteLogin?startURL=' + encodeURIComponent(startURL));
                });
                Handlebars.registerHelper('goToCart', function(pageKey) {
                    return new Handlebars.SafeString(CCRZ.pagevars.currSiteURL + 'ccrz__Cart?cartId=' + CCRZ.pagevars.currentCartID +  getCSRQueryString());
                });
                //return link to custom site form given pageKey
                Handlebars.registerHelper('goToCCPage_PageKey', function(pageKey) {
                    return new Handlebars.SafeString(CCRZ.pagevars.currSiteURL + 'ccrz__CCPage?pageKey=' + pageKey +  getCSRQueryString());
                });
                Handlebars.registerHelper('goToCategory', function(categoryId){
                    return new Handlebars.SafeString(CCRZ.pagevars.currSiteURL + 'ccrz__ProductList?viewState=ListView&cartId=' + CCRZ.pagevars.currentCartID + '&categoryId=' + categoryId + getCSRQueryString());
                });
                Handlebars.registerHelper('gridCalc', function(numOfSections) {
                    return 12/Math.floor(numOfSections);
                });
                Handlebars.registerHelper('replaceUnder', function(val) {
                    if(!_.isUndefined(val)){
                        return val.replace(' ', '_');
                    }
                });
                Handlebars.registerHelper('groupProductsCarouselItems', function(num1, num2, place) {
                    if(num2==1){
                        return 'true';
                    }
                    else{
                        if (place=="begin" && (num1==0 || (num1!=1 && (num1+1)%(num2)==1))) return 'true';
                        else if (place=="end" && ((num1+1)%num2==0)) return 'true';
                    }
                });
                
                Handlebars.registerHelper('isReprintProduct', function(isReprint, _this,options){
                    let subProdTerms = (_this.subProdTerms && _this.subProdTerms.length >0)?_this.subProdTerms:[];
                    let sptList = (subProdTerms.length <= 0)?subProdTerms:subProdTerms.filter(function(spt){
                        return spt.KFB2BSptType != 'Reprint';
                    });
                    if(!CCRZ.pagevars.isGuest && sptList.length <=0 && _this.isSubscriptionAvailable){
                        return options.inverse(this);
                    } 
                    return options.fn(this);
                });
                
                Handlebars.registerHelper('getSearchText', function(){
                    let searchText = typeof CCRZ.pagevars.queryParams.searchText == 'undefined'?"":CCRZ.pagevars.queryParams.searchText;
                    return searchText;
                });
                Handlebars.registerHelper('isCart', function(options){
                    if(CCRZ.pagevars.remoteContext.currentPageName == 'ccrz__Cart' || CCRZ.pagevars.remoteContext.currentPageName == 'ccrz__CheckoutNew'){
                    return options.fn(this);
                    } 
                    return options.inverse(this);
                });
                Handlebars.registerHelper('isGuest', function(options){
                    if(CCRZ.pagevars.isGuest){
                    return options.fn(this);
                    } 
                    return options.inverse(this);
                });
                Handlebars.registerHelper('getProductFamily', function(){
                    if(CCRZ.productDetailModel && CCRZ.productDetailModel.attributes.product && CCRZ.productDetailModel.attributes.product.prodBean
                        && CCRZ.productDetailModel.attributes.product.prodBean.KFB2BProductFamily){
                            if(CCRZ.productDetailModel.attributes.product.prodBean.KFB2BProductFamily == 'Columns and Features'){
                                return 'Columns';
                            } else if(CCRZ.productDetailModel.attributes.product.prodBean.KFB2BProductFamily == 'Puzzles and Games'){
                                return 'Puzzles';
                            } else {
                                return CCRZ.productDetailModel.attributes.product.prodBean.KFB2BProductFamily.replace(' ', '_');
                            }
                    }
                    return '';
                });
                Handlebars.registerHelper('isStaticPDP', function(options){
                    if(CCRZ.productDetailModel && CCRZ.productDetailModel.attributes.product && CCRZ.productDetailModel.attributes.product.prodBean
                        && CCRZ.productDetailModel.attributes.product.prodBean.KFB2BIsStaticPage){
                        return options.fn(this);
                    }
                    return options.inverse(this);
                });
                Handlebars.registerHelper('isNonAccredited', function(options){
                    if(CCRZ.pagevars.isNonAccredited){
                        return options.fn(this);
                    } 
                    return options.inverse(this);
                });
                Handlebars.registerHelper('ifAvail', function(_this, options){
                    return (typeof _this.product == 'undefined'?options.inverse(this):options.fn(this));
                });
                Handlebars.registerHelper('isAuthorAvail', function(options){
                    return (typeof CCRZ.productDetailModel.attributes.authorList != 'undefined'?(CCRZ.productDetailModel.attributes.authorList.length > 0?options.fn(this):options.inverse(this)):options.inverse(this));
                });
                Handlebars.registerHelper('isCharactersAvail', function(options){
                    return (typeof CCRZ.productDetailModel.attributes.characterList != 'undefined'?(CCRZ.productDetailModel.attributes.characterList.length > 0?options.fn(this):options.inverse(this)):options.inverse(this));
                });
                Handlebars.registerHelper('getAuthorList', function(){
                    return (typeof CCRZ.productDetailModel.attributes.authorList != 'undefined'?CCRZ.productDetailModel.attributes.authorList:[]);
                });
                Handlebars.registerHelper('getCharacterList', function(){
                    return (typeof CCRZ.productDetailModel.attributes.characterList != 'undefined'?CCRZ.productDetailModel.attributes.characterList:[]);
                });
                Handlebars.registerHelper('ifNotProdSub', function(options){
                    return (CCRZ.productDetailModel.attributes.product.prodBean.isSubscriptionAvailable?(CCRZ.productDetailModel.attributes.product.prodBean.cartData.isActiveSubsc?options.inverse(this):options.fn(this)):options.inverse(this));
                });
                Handlebars.registerHelper('ifProdDisable', function(minQuantity, options){
                    if(minQuantity > 1){
                        return options.fn(this);
                    }
                    return options.inverse(this);
                });
                Handlebars.registerHelper('getBillingAddress', function(address, fieldName){
                    let blnAddress = '';
                    if(address){
                        blnAddress = address;
                        switch(fieldName){
                            case 'street':
                                return (blnAddress.address1?unescape(blnAddress.address1):'');
                                break;
                            case 'city':
                                return (blnAddress.city?blnAddress.city:'');
                                break;
                            case 'state':
                                return (blnAddress.state?blnAddress.state:'');
                                break;
                            case 'postalCode':
                                return (blnAddress.postalCode?blnAddress.postalCode:'');
                                break;
                            case 'country':
                                return (blnAddress.country?blnAddress.country:'');
                                break;
                            case 'companyName':
                                return (blnAddress.companyName?blnAddress.companyName:'');
                                break;
                            default:
                                return '';
                        }
                    } else {
                        return '';
                    }
                });
                Handlebars.registerHelper('getBillingAddressByField', function(fieldName){
                    let blnAddress = '';
                    if(CCRZ.cartCheckoutModel && CCRZ.cartCheckoutModel.attributes.billingAddress){
                        blnAddress = CCRZ.cartCheckoutModel.attributes.billingAddress;
                        switch(fieldName){
                            case 'street':
                                return (blnAddress.address1?unescape(blnAddress.address1):'');
                                break;
                            case 'city':
                                return (blnAddress.city?blnAddress.city:'');
                                break;
                            case 'state':
                                return (blnAddress.state?blnAddress.state:'');
                                break;
                            case 'postalCode':
                                return (blnAddress.postalCode?blnAddress.postalCode:'');
                                break;
                            case 'country':
                                return (blnAddress.country?blnAddress.country:'');
                                break;
                            default:
                                return '';
                        }
                    } else {
                        return '';
                    }
                });
                Handlebars.registerHelper('getUserEmail', function(){
                    return ((CCRZ.currentUser && CCRZ.currentUser.Email)?CCRZ.currentUser.Email:'');
                });
                Handlebars.registerHelper('getUserPhone', function(){
                    return ((CCRZ.currentUser && CCRZ.currentUser.Phone)?CCRZ.currentUser.Phone:'');
                });
                Handlebars.registerHelper('getUserFirstName', function(){
                    return ((CCRZ.currentUser && CCRZ.currentUser.FirstName)?CCRZ.currentUser.FirstName:'');
                });
                Handlebars.registerHelper('getUserLastName', function(){
                    return ((CCRZ.currentUser && CCRZ.currentUser.LastName)?CCRZ.currentUser.LastName:'');
                });
                Handlebars.registerHelper('getCurrentDate', function(){
                    return util.formatDate(new Date(), 'mm/dd/yyyy');
                });
                Handlebars.registerHelper('getDateByDays', function(days){
                    let date = new Date();
                    date.setDate(date.getDate() + days)
                    return util.formatDate(date, 'mm/dd/yyyy');
                });
                Handlebars.registerHelper('getSubscDates', function(frequency, frequencyUOM){
                    let currentDate = new Date();
                    let endDate = (frequencyUOM == 'Monthly'? currentDate.setMonth(currentDate.getMonth() + frequency):currentDate.setFullYear(currentDate.getFullYear() + frequency));
                    endDate = new Date(endDate);
                    endDate = endDate.setDate(endDate.getDate() - 1);
                    return util.formatDate(new Date(endDate), 'mm/dd/yyyy');
                });
                Handlebars.registerHelper('isReprint', function(options){
                    if(typeof CCRZ.pagevars.queryParams.searchText != 'undefined'){
                        return options.fn(this);
                    } else {
                        return options.inverse(this);
                    }
                });
                Handlebars.registerHelper('getPDPDesc', function(){
                    let longDesc = '';
                    if(!_.isUndefined(CCRZ.productDetailModel) && !_.isUndefined(CCRZ.productDetailModel.attributes.product.prodBean) 
                    && !_.isUndefined(CCRZ.productDetailModel.attributes.product.prodBean.longDesc)){
                        longDesc = CCRZ.productDetailModel.attributes.product.prodBean.longDesc;
                        let e = document.createElement('div');
                        e.innerHTML = CCRZ.productDetailModel.attributes.product.prodBean.longDesc;
                        if(e.innerHTML != '' && e.childNodes[0].nodeValue != null){
                            longDesc = e.childNodes[0].nodeValue;
                            e.innerHTML = e.childNodes[0].nodeValue;
                            if(e.innerHTML != '' && e.childNodes[0].nodeValue != null){
                                longDesc = e.childNodes[0].nodeValue;
                            }
                        }
                    } else {
                        longDesc = '';
                    }
                    return longDesc;
                });
                Handlebars.registerHelper('isReprintPDP', function(options){
                    let isReprint = (CCRZ.pagevars.queryParams.isReprint == 'true'?true:false);
                    return isReprint?options.fn(this):options.inverse(this);
                });
                Handlebars.registerHelper("each_cat", function(limit, options) {
                    let ret = "", context = CCRZ.data.categories;
                    if(!_.isUndefined(context)){
                        limit = (limit < context.length?limit:context.length);
                        for (let i = 0, j = limit; i < j; i++) {
                            ret = ret + options.fn(context[i]);
                        }
                    }
                    return ret;
                });
                Handlebars.registerHelper("each_range", function(context, index, limit, options) {
                    let ret = "";
                    limit = (context.length > limit?limit:context.length);
                    for (let i = index, j = limit; i < j; i++) {
                      ret = ret + options.fn(context[i]);
                    }
                    return ret;
                });
                Handlebars.registerHelper('isAutoRenew', function(key, _this, options){
                    if(CCRZ.cartCheckoutModel){
                        if(key == 'po'){
                            if(CCRZ.pagevars.pageConfig.isTrue('CO.showpo')){
                                if(CCRZ.cartCheckoutModel.attributes.accountR.DCMApprovedForCredit && !CCRZ.pagevars.isNonAccredited){
                                    return options.fn(this);
                                } else {
                                    return options.inverse(this);
                                }
                            } else {
                                return options.inverse(this);
                            }
                        } else if(key == 'pod'){
                            if(CCRZ.pagevars.pageConfig.isTrue('CO.showcc')){
                                return options.fn(this);
                            } else if(CCRZ.cartCheckoutModel.attributes.accountR.DCMCreditCardPaymentAvailable){
                                return options.fn(this);
                            } else {
                                return options.inverse(this);
                            }
                        }
                    }
                    return options.fn(this);
                });
                Handlebars.registerHelper('getImageURL', function(imagePath){
                    return CCRZ.pagevars.themeBaseURL + imagePath;
                });
                
                Handlebars.registerHelper('getImageStatic', function(media, type, staticResourceName, filePath){
                    if((!_.isUndefined(media) && (!_.isUndefined(media.ccrz__StaticResourceName__c) || !_.isUndefined(media.staticResourceName))) || !_.isUndefined(staticResourceName)){
                        if(type == 'category'){
                            return window.location.origin + CCRZ.pagevars.staticResourceMap[media.ccrz__StaticResourceName__c] + '/' + media.ccrz__FilePath__c;
                        } else if(type == 'spotlight'){
                            let resourceName = (media.staticResourceName).replace('c__', '');
                            return window.location.origin + CCRZ.pagevars.staticResourceMap[resourceName] + '/' + media.filePath;
                        } else if(type == 'static'){
                            let resourceName = (staticResourceName).replace('c__', '');
                            return window.location.origin + CCRZ.pagevars.staticResourceMap[resourceName] + '/' + filePath;
                        }
                    } else {
                        filePath = 'images/image-not-available.png';
                        return CCRZ.pagevars.themeBaseURL + filePath
                    }
                });
                Handlebars.registerHelper('getSubTotal', function(subTotal, hasCoupon, items, type){
                    if(hasCoupon){
                        if(items && items.length > 0){
                            for(let i = 0; i < items.length; i++){
                                switch(type) {
                                    case 'cart':
                                        if(items[i].attributes.productType == 'Coupon'){
                                            subTotal += -(items[i].attributes.itemTotal);
                                        } else {
                                            subTotal += (items[i].attributes.baseDiscount?items[i].attributes.baseDiscount:0);
                                        }
                                        break;
                                    case 'review':
                                        if(items[i].productType == 'Coupon'){
                                            subTotal += -(items[i].itemTotal);
                                        } else {
                                            subTotal += (items[i].baseDiscount?items[i].baseDiscount:0);
                                        }
                                        break;
                                    case 'order':
                                        if(items[i].productType == 'Coupon'){
                                            subTotal += -(items[i].itemTotal);
                                        } else {
                                            subTotal += (items[i].absoluteDiscount?items[i].absoluteDiscount:0);
                                        }
                                        break;
                                    default:
                                        return '';
                                }
                            }
                        }
                        
                    }
                    return subTotal;
                }); 
                Handlebars.registerHelper('ifProdMax', function(prodBean, options){
                    /*if(typeof prodBean != 'undefined' && prodBean.tierList != 'undefined' && prodBean.tierList.length > 0){
                        for(let i = 0; i < prodBean.tierList.length; i++){
                            if(prodBean.tierList[i].currencyCode == CCRZ.pagevars.remoteContext.userIsoCode && prodBean.cartData.prodCartQuantity >= prodBean.tierList[i].maxQuantity){
                                return options.inverse(this);
                            }
                        }
                    } else {
                        let tierList = CCRZ.productDetailModel.attributes.product.prodBean.tierList, cartData = CCRZ.productDetailModel.attributes.product.prodBean.cartData;
                        if(typeof cartData != 'undefined'){
                            for(let i = 0; i < tierList.length; i++){
                                if(tierList[i].currencyCode == CCRZ.pagevars.remoteContext.userIsoCode && cartData.prodCartQuantity >= tierList[i].maxQuantity){
                                    return options.inverse(this);
                                }
                            }
                        } else {
                            return options.inverse(this);
                        }
                    }*/
                    return options.fn(this);
                });
                Handlebars.registerHelper('quantityCheck', function(minQty,maxQty, options){
                    if(minQty == maxQty || minQty > 1){
                    return options.fn(this);
                    }
                    return options.inverse(this);
                });
                
                Handlebars.registerHelper('isOrderDetail', function(options){
                    if(window.location.href.indexOf('ccrz__OrderView') != -1){
                        return options.fn(this);
                    }
                    return options.inverse(this);
                });
                Handlebars.registerHelper('checkCurrentCurrency', function(currency, options){
                    if(currency == CCRZ.pagevars.remoteContext.userIsoCode){
                    return options.fn(this);
                    }
                    return options.inverse(this);
                });
                Handlebars.registerHelper('divideAndRoundUp', function(num1, num2) {
                    return Math.ceil(num1/num2);
                });
                Handlebars.registerHelper('loop', function(n, block) {
                    var accum = '';
                    for(var i = 0; i < n; ++i)
                        accum += block.fn(i);
                    return accum;
                });
                Handlebars.registerHelper('pageLabelMap', function(labelName) {
                    var params = Array.prototype.slice.call(arguments, 1);
                    var labelSymbol = "EL#";

                    var retLabel;
                    if (labelName) {
                        if (sessionStorage.getItem("editModeOn")!='true' && CCRZ.pagevars.pageLabels && !_.isUndefined(CCRZ.pagevars.pageLabels[labelName])) {
                            retLabel = CCRZ.pagevars.pageLabels[labelName];
                            if (_.isNull(retLabel)) {
                                retLabel = '';
                            } else if (!_.isUndefined(params) && params.length > 0) {
                                retLabel = new Handlebars.SafeString(substitute(retLabel, params));
                            } else {
                                retLabel = new Handlebars.SafeString(retLabel);
                            }
                        } else {
                            if (sessionStorage.getItem("editModeOn")=='true') {
                                retLabel = CCRZ.pagevars.pageLabels[labelName];
                                    if (_.isNull(retLabel)) {
                                        retLabel = '';
                                    } else if (!_.isUndefined(params) && params.length > 0) {
                                        var editableLabel = labelSymbol + ' [' + labelName + '] ' + retLabel;
                                        retLabel = new Handlebars.SafeString(substitute(editableLabel, params));
                                    } else {
                                        var editableLabel = labelSymbol + ' [' + labelName + '] ' + retLabel;
                                        retLabel = new Handlebars.SafeString(editableLabel);
                                    }
                            } else {
                                retLabel = labelName;
                            }
                        }
                    }
                    if (!retLabel) {
                        retLabel = '';
                    }
                    return retLabel;
                });
                Handlebars.registerHelper('pageLabelPrefixMap', function(prefix, labelName) {
                    return Handlebars.helpers.pageLabelMap(prefix+labelName);
                });
                Handlebars.registerHelper('ifCurrentTheme', function(themeValue) {
                    var theme = (CCRZ.pagevars.themeBaseURL).split('/');
                    return theme[theme.length-2] == themeValue;
                });
                Handlebars.registerHelper('textToHTML', function(str) {
                    return str.replace(/&amp;/g, "&").replace(/&lt;/g, "<").replace(/&gt;/g, ">");
                });
                Handlebars.registerHelper('isCSRMode', function(options) {
                    if (CCRZ && CCRZ.pagevars && CCRZ.pagevars.portalUserId != ''){
                        return options.fn(this);
                    } else {
                        return options.inverse(this);
                    }
                });
                // Removing helper userName
                /*Handlebars.registerHelper('userName', function(options){
                    return r('user').get('user') ? r('user').get('user').FirstName : '';
                });*/
                Handlebars.registerHelper('getTaxesAmountFromJSON', function(cartData){
                    var taxesAmount = 0;
                    if(cartData){
                        var taxesJSON = cartData.cccustTaxesList;
                        if(taxesJSON){
                            var taxesList = JSON.parse($.parseHTML(taxesJSON)[0].data);
                            if(taxesList){
                                _.each(taxesList, function(tax){
                                    taxesAmount += tax['Amount'];
                                })
                            }
                        }
                    }
                    return taxesAmount;
                });
                Handlebars.registerHelper('getTaxesAmountFromJSONByDescription', function(cartData,desc){
                    var taxesAmount = 0;
                    if(cartData){
                        var taxesJSON = cartData.cccustTaxesList;
                        if(taxesJSON){
                            var taxesList = JSON.parse($.parseHTML(taxesJSON)[0].data);
                            if(taxesList){
                                _.each(taxesList, function(tax){
                                    if(desc == tax['TaxDescr']) taxesAmount += tax['Amount'];
                                })
                            }
                        }
                    }
                    return taxesAmount;
                });
                Handlebars.registerHelper('containsTaxesByDescription', function(cartData,desc){
                    if(cartData){
                        var taxesJSON = cartData.cccustTaxesList;
                        if(taxesJSON){
                            var taxesList = JSON.parse($.parseHTML(taxesJSON)[0].data);
                            if(taxesList){
                                return _.some(taxesList,function(tax){
                                     return desc == tax['TaxDescr']
                                 })
                            }
                        }
                    }
                    return false;
                });
                Handlebars.registerHelper('removeAfterSign', function(text, sign){
                    return text ? text.substr(0, text.indexOf(sign) + 1) : '';
                });
                Handlebars.registerHelper('checkoutStepNumber', function(index){
                    return index + 1;
                });
                Handlebars.registerHelper('equalsOrLower',  function(index, parentIndex, options){
                    if(index <= parentIndex) return options.fn(this);
                    return options.inverse(this);
                });
                Handlebars.registerHelper('getViewState', function(){
                    return CCRZ.getQueryParam('viewState');
                });
                Handlebars.registerHelper('normalizeShippingMethod', function(shipMethod){
                    return shipMethod.replace('null - ', '');
                });
                Handlebars.registerHelper('cartDiscountTotal', function(cartItems){
                    var totalDiscount = 0;
                    if(cartItems){
                        _.each(cartItems, function(item){
                            if(item.attributes && item.attributes.absoluteDiscount){
                                totalDiscount += item.attributes.absoluteDiscount;
                            }else if(item.absoluteDiscount){
                                totalDiscount += item.absoluteDiscount;
                            }
                        });
                    }
                    return totalDiscount;
                });
                Handlebars.registerHelper('getFooterLink', function(url) {
                    var sign = '?',
                        cartParam = '',
                        CSRQueryString = getCSRQueryString(),
                        containsQuestionMark = false,
                        storefront = CCRZ.pagevars.remoteContext.storefront;

                    if(url.indexOf('www') === 0 || url.indexOf('http') === 0 || url.indexOf('https') === 0) {
                       return url;
                    }
                    if(url.indexOf("?") >= 0) {
                        sign = '&';
                        containsQuestionMark = true;
                    }
                    if(CCRZ.pagevars.currentCartID != null){
                        cartParam = sign + 'cartId=' + CCRZ.pagevars.currentCartID;
                        containsQuestionMark = true;
                    }
                    if(containsQuestionMark == false && CSRQueryString.indexOf('&') == 0) {
                        CSRQueryString = CSRQueryString.replace('&', '?');
                    }
                   return url + cartParam + CSRQueryString;
                });
                Handlebars.registerHelper('getAbsoluteValue', function(number){
                    return Math.abs(number);
                });
                Handlebars.registerHelper('checkCouponInCartItems', function(cartItemList){
                    var isCouponInCart = null;
                    var coupon = 'Coupon';

                    isCouponInCart = _.find(cartItemList, function(element){
                        return element.cartItemType == coupon || element.productType == coupon;
                    });

                    return isCouponInCart ? true : false;
                });
                Handlebars.registerHelper('getStoreFrontName', function(){
                    return CCRZ.pagevars.remoteContext.storefront;
                });
                initialized = true;
            }
        }
    }
});