/**
 *  Namespace for functions that can can be used in several views/namespaces
 *  @module util
 */
d('util',['util'], function(UTIL) {

    var currency_symbols = {
        'USD': '$', // US Dollar
        'EUR': '€', // Euro
        'CRC': '₡', // Costa Rican Colón
        'GBP': '£', // British Pound Sterling
        'ILS': '₪', // Israeli New Sheqel
        'INR': '₹', // Indian Rupee
        'JPY': '¥', // Japanese Yen
        'KRW': '₩', // South Korean Won
        'NGN': '₦', // Nigerian Naira
        'PHP': '₱', // Philippine Peso
        'PLN': 'zł', // Polish Zloty
        'PYG': '₲', // Paraguayan Guarani
        'THB': '฿', // Thai Baht
        'UAH': '₴', // Ukrainian Hryvnia
        'VND': '₫', // Vietnamese Dong
    };
    function formatCurrency(amount){
        let numFormat = new Intl.NumberFormat(CCRZ.pagevars.remoteContext.userLocale.replace('_','-'), { 
            style: 'currency', 
            currency: CCRZ.pagevars.remoteContext.userIsoCode, 
            minimumFractionDigits: 2, 
        }); 
        return numFormat.format(amount);
    }
    return {
        /** IMPLEMENTATION OF VALIDATION WITH ERROR LABELS
        1.Input/textarea must be inside <div data-error="Error label"></div> section.
        2.Inputs/textareas must contain classes: req_field, 'inputClass' (own unique name), col-md-12.
        3.Email and phone are optional parameters. Their fields require <div class="col-md-12 incorrectEmail"></div> (or 'incorrectPhone') section below input.
        4.Add attribute disabled="disabled" to save button.
        5.Use functions for:
            ON EVENT (e.g. keyup) => validateFields
            OTHER (e.g. view refresh) => validateAndDisable
        */
        /**
        * Function that check if fields are proper or not empty and disable button until inputs are correct
          Email and phone fields are optional.
        */
        validateFields: function(e, view, inputClass, button, email, phone){
            validateEmptyFields(e, validateAndDisable(view, inputClass, button, email, phone));
        },
        /**
        * Function that check if fields are proper and disable button until inputs are correct
        Email and phone fields are optional.
        */
        validateAndDisable : function(view, inputClass, button, email, phone){
            validateAndDisable(view, inputClass, button, email, phone);
        },
        /**
        * Function that check if inputs are empty
        */
        validateEmptyFields : function(e, callback){
            validateEmptyFields (e, callback);
        },
        /**
        * Function that check if phone is valid
        */
        validatePhone : function(value) {
            return validatePhone(value);
        },
        /**
        * Function that check if email is valid
        */
        validateEmail : function(email){
            return validateEmail(email);
        },
        /**
        * Function that check if phone number from object is valid and show error label
        */
        validatePhoneFromObject : function(value, callback) {
            validatePhoneFromObject(value, callback);
        },
        /**
        * Function that check if email address from object is valid and show error label
        */
        validateEmailFromObject : function(email, callback){
            return validateEmailFromObject(email, callback);
        },
        /**
        * Function that check if email address from object is valid and show error label
        */
        validateDatepicker : function(date){
            return validateDatepicker(date);
        },
        /**
        * Function that remove readonly attribute for HTML tag
        */
        removeReadonly : function(date){
            return removeReadonly(date);
        },
        /**
        * Functions that shows and hides loading cover
        */
        loadingOn : function() {

        },
        loadingOff : function() {

        },
        /**
        * Function that finds dynamic view by its name
        */
        findDynamicView : function(viewName) {
            var dynamicView = _.findWhere(CCRZ.dynamicCustomView, { viewName : viewName });
            return dynamicView;
        },
        findCheckoutView : function(viewName) {
            var viewsArray = CCRZ.cartCheckoutView.subViewArray;

            var subView = _.filter(viewsArray, function(item){
                var view = item.view;
                if (view.viewName === viewName) {
                    return view;
                }
            }, viewName);

            return subView[0].view;
        },
        /**
        * Function that finds LLI Checkout subView by name
        */
        findLLIView : function(viewName) {
            var viewsArray = CCRZ.LLIcartCheckoutView.subViewArray;

            var subView = _.filter(viewsArray, function(item){
                var view = item.view;
                if (view.viewName === viewName) {
                    return view;
                }
            }, viewName);

            return subView[0].view;
        },
        rotateArrow: function(event){
            var icon = $(event.currentTarget).find(".icon-toggle i");
            var toggleableElement =  $(event.currentTarget).data("target");

            $(toggleableElement).on("hidden.bs.collapse", function(){
                icon.addClass('fa-caret-down').removeClass('fa-caret-up');
            });
            $(toggleableElement).on("shown.bs.collapse", function(){
                icon.removeClass('fa-caret-down').addClass('fa-caret-up');
            });
        },
        //Function to format currency
        formatCurrency: function(amount){
            let numFormat = new Intl.NumberFormat(CCRZ.pagevars.remoteContext.userLocale.replace('_','-'), { 
                style: 'currency', 
                currency: CCRZ.pagevars.remoteContext.userIsoCode, 
                minimumFractionDigits: 2, 
            }); 
            return numFormat.format(amount);
        },
        // Validate and format subscription name 
        validateDisplayName: function(item, _this){
            let displayName = '';
            if(item.modifierSubscriptions){
                let installCountTotal = 0;
                item.modifierSubscriptions.forEach(function(modifier, index){
                    displayName += (index == 0)?'First ': ' Next ';
                    displayName += (modifier.installmentCount).toString();
                    displayName += (item.installmentFrequencyUOM == 'Monthly')?' Months:': ' Years:';
                    displayName += (formatCurrency(modifier.productPrice)).toString(); 
                    displayName += ' and ';
                    installCountTotal += modifier.installmentCount;
                });
                displayName += ' Last ' ;
                displayName += (item.installmentCount - installCountTotal).toString() ;
                displayName += (item.installmentFrequencyUOM == 'Monthly')?' Months:': ' Years:';
                displayName += (formatCurrency(item.productPrice)).toString();
            } else {
                displayName = formatCurrency(item.productPrice);
            }
            _this.model.set('displayName', displayName);
        },
        //Go To Cart by Id
        goToLogin: function(){
            return new Handlebars.SafeString(CCRZ.pagevars.currSiteURL + 'ccrz__CCSiteLogin?registrationSuccess=true' +  getCSRQueryString());
        },
        //Go To Cart by Id
        goToCart: function(cartId){
            return new Handlebars.SafeString(CCRZ.pagevars.currSiteURL + 'ccrz__Cart?cartId=' + cartId +  getCSRQueryString());
        },
        //Function to return the tier price based off quantity
        getTierPrice: function(tierPricingList, quantity){
            for(let i = 0; i < tierPricingList.length; i++){
                if(tierPricingList[i].maxQuantity == null){
                    if(quantity >= tierPricingList[i].minQuantity){
                        return tierPricingList[i].listPrice;
                    }
                } else if(quantity >= tierPricingList[i].minQuantity && quantity <= tierPricingList[i].maxQuantity){
                    return tierPricingList[i].listPrice;
                }
            }
        },
        // End Get Tier Price 
        /**
        * Function that checks if value is integer or decimal number
        */
        validateDecimalAndIntegerField : function(event){
            var value = $(event.target).val();
            if(event.which < 45 || event.which > 58 || event.which == 47 ){
                return false;
            } // prevent if not number/dot
            if(event.which == 46 && value.indexOf('.') != -1){
                return false;
            } // prevent if already dot
            if(event.which == 45 && value.indexOf('-') != -1){
                return false;
            } // prevent if already dot
            if(event.which == 45 && value.length>0){
                event.preventDefault();
            } // prevent if already -
            return true;
        },
        checkIfFieldIsNotEmpty : function(input){
            var value = input.val().trim();
            return (value != "") ? true : false;
        },
        scrollToTop:function() {
           $('html, body').animate({
                 scrollTop: 0
           }, 500);
        },
        validateAllFields: function(inputClass, emailId, phoneId){
            var inputs = $("."+inputClass+"");
            var validate = true;
            _.each(inputs, function(e){
                var target = e.target === undefined ? $(e) : $(e.target);
                validate = validate && validateEmptyFields(e);
                validateEmptyFields(e, function(result){
                    
                });
                if(emailId){
                    if(target.attr('id') == emailId){
                        validate = validate && validateEmailFromObject(target);
                        validateEmailFromObject(target, function(result){
                    
                        });
                    }
                }
                if(phoneId){
                    if(target.attr('id') == phoneId){
                        validate = validate && validatePhoneFromObject(target);
                        validatePhoneFromObject(target, function(result){
                    
                        });
                    }
                }
            });
            return validate;
        },
        keepIfCollapsed: function (e, cookieName){
            var isCollapsed = $(e.currentTarget).find('.icon-toggle .fa').hasClass('fa-caret-up'),
                daysToExpire = 180;
            setCookie(cookieName, String(isCollapsed), daysToExpire);
        },
        collapseOnRender: function(view, cookieName){
            var isCollapsed = getCookie(cookieName),
                panelCollapse = view.$el.find('.panel-collapse'),
                iconCollapse = view.$el.find('.icon-toggle .fa');
            if (isCollapsed == "true"){
                panelCollapse.removeClass('in');
                iconCollapse.removeClass('fa-caret-up').addClass('fa-caret-down').attr('aria-expanded','false');
            }
        },
        convertStringToDate : function(dateToConvert){
            return convertStringToDate(dateToConvert);
        },
        formatDate : function(date,dateFormat){
            var year = date.getFullYear();
            var month = date.getMonth() + 1;

            if(month < 10){
                month = '0'+month;
            }
            var day = date.getDate();
            if(day < 10){
                day = '0'+day;
            }
            switch(dateFormat){
                case 'yyyy/mm/dd':
                    return year+'/'+month+'/'+day;
                case 'dd/mm/yyyy':
                    return day+'/'+month+'/'+year;
                default:
                    return month+'/'+day+'/'+year;
            }
        },
        isMobile : function(){
            var screensize = $(window).width();
            var maxDeviceWidth = 768; //max device width in px

            return (screensize < maxDeviceWidth) ? true : false;;
        },
        calculateSubscriptionPrices: function(items, view, type){
            let monthlyPrice = 0, yearlyPrice = 0, weeklyPrice = 0, isRecurring = false;
            items.forEach(function(item){
                if(type == 'cart'){
                    if(typeof item.attributes.subProdTerm != 'undefined' && item.attributes.DCMAutoRenew){
                        if(item.attributes.subProdTermR.installmentFrequencyUOM == 'Monthly' && !item.attributes.product.DCMB2BIsTrialProduct){
                            monthlyPrice += item.attributes.itemTotal;
                        } else if(item.attributes.subProdTermR.installmentFrequencyUOM == 'Yearly' && !item.attributes.product.DCMB2BIsTrialProduct){
                            yearlyPrice += item.attributes.itemTotal;
                        } else if(item.attributes.subProdTermR.installmentFrequencyUOM == 'Weekly' && !item.attributes.product.DCMB2BIsTrialProduct){
                            weeklyPrice += item.attributes.itemTotal;
                        }
                    }
                } else if(type == 'review'){
                    if(item.mockProduct.isSubscriptionAvailable && item.subProdterm && item.DCMAutoRenew){
                        if(item.subProdTermR.installmentFrequencyUOM == 'Monthly' && !item.mockProduct.DCMB2BIsTrialProduct){
                            monthlyPrice += item.itemTotal;
                        } else if(item.subProdTermR.installmentFrequencyUOM == 'Yearly' && !item.mockProduct.DCMB2BIsTrialProduct){
                            yearlyPrice += item.itemTotal;
                        } else if(item.subProdTermR.installmentFrequencyUOM == 'Weekly' && !item.mockProduct.DCMB2BIsTrialProduct){
                            weeklyPrice += item.itemTotal;
                        }
                    }
                } else if(type == 'order'){
                    if(item.mockProduct.isSubscriptionAvailable && item.DCMAutoRenew){
                        if(item.installmentFrequencyUOM == 'Monthly' && !item.mockProduct.DCMB2BIsTrialProduct){
                            monthlyPrice += item.itemTotal;
                        } else if(item.installmentFrequencyUOM == 'Yearly' && !item.mockProduct.DCMB2BIsTrialProduct){
                            yearlyPrice += item.itemTotal;
                        } else if(item.installmentFrequencyUOM == 'Weekly' && !item.mockProduct.DCMB2BIsTrialProduct){
                            weeklyPrice += item.itemTotal;
                        }
                    }
                }
            });
            if(monthlyPrice > 0 || yearlyPrice > 0 || weeklyPrice > 0){
                isRecurring = true;
            }
            view.model.set('monthlyPrice', monthlyPrice);
            view.model.set('yearlyPrice', yearlyPrice);
            view.model.set('weeklyPrice', weeklyPrice);
            view.model.set('recurring', isRecurring);
         },
        validateAllFieldTypesById: function(inputClass) {
            let check = true;
            var input = $(inputClass), select = $('.inputVal select');
            for (var i = 0; i < input.length; i++) {
                if (this.validate(input[i]) == false) {
                    this.showValidate(input[i]);
                    check = false;
                }
            }
            for (var i = 0; i < select.length; i++) {
                if (this.validateSelect(select[i]) == false) {
                    this.showValidate(select[i]);
                    check = false;
                }
            }
            return check;
        },
        parseImage : function(path){
            let merlinBaseUrl = 'https://kingfeatures.merlinone.net/MX/';
            if (path==null || path.length==0){
                //console.log('Most provide a local path to transform.');
                throw 'Most provide a local path to transform.'; // Replace Windows-style backslashes with URL-style forward slashes.
            }
            path = path.replace(/\\/g,'/');
            // Look for the thumbs portion of the string, and retain only that. if (idx<0) path = path.substring(idx); // Ensure that the base URL has a trailing slash.
            var idx = path.toLowerCase().indexOf('vidthumbs');
            idx = (idx == -1)?path.toLowerCase().indexOf('thumbs'):idx;
            idx = (idx == -1)?path.toLowerCase().indexOf('iptc'):idx;

            if (merlinBaseUrl.length==0 || merlinBaseUrl[merlinBaseUrl.length-1]!='/'){
                merlinBaseUrl += '/'; 
            }
            return merlinBaseUrl + path.substring(idx);
        },
        isAlphaNumeric : function(str){
            let regex = new RegExp("^[A-Za-z0-9]+$");
            if (regex.test(str)) {
                return true;
            }
            return false;
        },
        validateAllFieldTypes: function() {
            let check = true;
            var input = $('.inputVal input'), select = $('.inputVal select');
            for (var i = 0; i < input.length; i++) {
                if (this.validate(input[i]) == false) {
                    this.showValidate(input[i]);
                    check = false;
                }
            }
            for (var i = 0; i < select.length; i++) {
                if (this.validateSelect(select[i]) == false) {
                    this.showValidate(select[i]);
                    check = false;
                }
            }
            return check;
        },
        validate: function(input) {
            let parent = $(input).parent()[0], validate = parent.dataset.default;
            if ($(input).attr('type') == 'email' || $(input).attr('name') == 'email') {
                if ($(input).val().trim() == '') {
                    $(parent).attr('data-validate', CCRZ.processPageLabelMap('KFB2B_all_missingEmail').toString());
                    return false;
                } else if ($(input).val().trim().match(/^([a-zA-Z0-9_\-\.]+)@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.)|(([a-zA-Z0-9\-]+\.)+))([a-zA-Z]{1,5}|[0-9]{1,3})(\]?)$/) == null) {
                    $(parent).attr('data-validate', CCRZ.processPageLabelMap('KFB2B_all_invalidEmail').toString());
                    return false;
                }
            } else if($(input).attr('type') == 'password'){
                if ($(input).val().trim() == '') {
                    return false;
                } else if ($(input).val().trim().match(/^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)[a-zA-Z\d]{8,}$/) == null) {
                    $(parent).attr('data-validate', 'Criteria not met');
                    if($(input).data('type') == 'register'){
                        $('.messagingSection-registerModal-Error').html('<span class="wordWrap">' + CCRZ.processPageLabelMap('KFB2B_all_passwordRequirementsTitle').toString() + '<br/>' + CCRZ.processPageLabelMap('KFB2B_all_passwordRequirements').toString() + '</span>');
                        $('.messagingSection-registerModal-Error').css('display', 'block');
                    } else if($(input).data('type') == 'login') {
                        $('.messagingSection-loginModal-Error').html('<span class="wordWrap">' + CCRZ.processPageLabelMap('KFB2B_all_passwordRequirementsTitle').toString() + '<br/>' + CCRZ.processPageLabelMap('KFB2B_all_passwordRequirements').toString() + '</span>');
                        $('.messagingSection-loginModal-Error').css('display', 'block');
                    }
                    return false;
                }
            } else {
                if ($(input).val().trim() == '') {
                    return false;
                }
            }
        },
        validateSelect: function(select) {
            if ($(select).val().trim() == '') {
                return false;
            }
        },
        showValidate: function(input) {
            var thisAlert = $(input).parent();

            $(thisAlert).addClass('alert-validate');
        },
        hideValidate: function(input) {
            var thisAlert = $(input).parent();
            $(thisAlert).removeClass('alert-validate');
        },
        currViewState : function(){
            return CCRZ.getQueryParam('viewState');
        },
        clearInput : function(e){
            e.target.value = '';
        },
        setDefaultQty : function(e){
            if (e.target.value == '') e.target.value = 1;
        },
        loadOverlay : function(container) {
            $(container).prepend(createOverlay());
            if (!CCRZ.disableAdaptive) {
                $("#overlay").css({
                    width: $(window).width() / 2,
                    height: $(window).height() / 2,
                    top: $(window).height() / 4,
                    left: $(window).width() / 4
                });
                $(".circleContainer").css({
                    top: $(window).height() / 5,
                    left: $(window).width() / 5
                });
            }
            $("#overlay").show();
        },
        unloadOverlay : function() {
            $("#overlay").remove();
        },
        removeWhitespaceFromInput : function(e, fieldId) {
            if (e.target.value.indexOf(' ')>-1) {
                var fieldValue = document.querySelector(fieldId).value;
                document.querySelector(fieldId).value = fieldValue.replace(/ /g,'');
            }
        },
        isValidField :function(elementId){
            if($.trim($("#"+elementId).val()) ===''){
                return false;
            }else{
                return true;
            }
        },
        getCurrencySymbol: function(currency_name){
            if(currency_symbols[currency_name]!==undefined) {
                return (currency_symbols[currency_name]);
            }
        },
        runHandleBarsUtils: function(){
            runHandleBarsUtils();
        },
        renderCommonHeaderAndFooter: function(cb){
            renderCommonHeaderAndFooter(cb);
        },
        hideLoader: function(){
            hideLoader();
        },
        showLoader: function(){
            showLoader();
        },
        disableScroll: function(){
            $('body').css('overflow', 'hidden');
        },
        enableScroll: function(){
            $('body').css('overflow', 'auto');
        },
        getNavigateURL: function(pageName, params){
            let url = CCRZ.pagevars.currSiteURL;
            if(pageName == 'subscriber'){
                url +='ccrz__CCPage?';
            } else if(pageName == 'pdp'){
                url +='ccrz__ProductDetails?';
            }  else if(pageName == 'plp'){
                url +='ccrz__ProductList?';
            }  else if(pageName == 'cart'){
                url +='ccrz__Cart?';
            }  else if(pageName == 'checkout'){
                url +='ccrz__CheckoutNew?';
            } else if(pageName == 'home'){
                url +='?';
            }
            for(let key in params){
                url += key + '=' + params[key] + '&'
            }
            if(params){
                url.slice(0,-1);
            }
            url += getCSRQueryString();
            return url;
        }
    }

    function createOverlay(){
        if (!CCRZ.disableAdaptive) {
            if (Modernizr.mq('only all')) {
                return "<div id='overlay'><div class='circleContainer'><div class='circle'></div><div class='circle1'></div></div></div>";
            } else {
                return "<div id='overlay'><div class='circleContainer'><img src='" + CCRZ.pagevars.themeBaseURL + "images/loading.gif' alt='loading...' /></div></div>";
            }
        } else {
            return "<div id='overlay' class='modal-backdrop fade in'></div>";
        }
    }
    function validateEmptyFields (e, callback) {
        var target = e.target === undefined ? $(e) : $(e.target),
            idTarget = target.attr("id"),
            value = target.val() ? target.val().trim() : "",
            keyCode = e.keyCode || e.which,
            tabKeyCode = 9,
            spaceKeyCode = 32,
            result = false;
        var className = target.attr('class');
        if (value == "" && keyCode == spaceKeyCode){
            target.val("");
        }
        if (value == "" & keyCode != tabKeyCode) {
            if(target.parent().find('button').length !=1){
                target.attr("placeholder","");
                if(target.closest('.form-group')){
                    target.closest('.form-group').addClass('input-error');
                }
                if(target.closest('.select-wrap')){
                    target.closest('.select-wrap').addClass('input-error');
                }
            }
            result = false;
        }
        else {
            if(className.indexOf('phone') >= 0){
                return validatePhoneFromObject(target, callback);
            }else if(className.indexOf('email') >= 0){
                return validateEmailFromObject(target, callback);
            }else{
                if(target.closest('.form-group')){
                    target.closest('.form-group').removeClass('input-error');
                }
                if(target.closest('.select-wrap')){
                    target.closest('.select-wrap').removeClass('input-error');
                }
            }
            result = true;
        }
        if(callback){
            callback(result);
        }
        return result;
    }

    function checkRequiredFields(view,inputClass){
        var v = view,
            reqFields = v.$el.find('.req_field').filter(inputClass),
            result = true;
        _.each(reqFields, function(e){
            if(e.value.trim() == ""){
                result=false;
            }
        });
        return result;
    }

    function validateAndDisable(view, inputClass, buttonClass, emailClass, phoneClass){
        var v = view;
        if (emailClass){
            var email = v.$el.find(emailClass);
        }
        if (phoneClass){
            var phone = v.$el.find(phoneClass);
        }
        if(buttonClass){
            if (checkRequiredFields(v,inputClass) && validateEmailFromObject(email) && validatePhoneFromObject(phone))
            {
                $(buttonClass).attr("disabled", false);
            }
            else
            {
                $(buttonClass).attr("disabled", true);
            }
        }
    }

     function validateEmail(email){
         var result = false,
            email_regex = /^[a-zA-Z0-9._-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,63}$/i;
         if(email_regex.test(email)){
             result = true;
         }
         return result;
     }

      function validatePhone(phone){
          var isValidPhone = false,
              regex =  /^[+]*[(]{0,1}[0-9]{1,4}[)]{0,1}[-\s\./0-9]*$/;
          var isValidPhone = phone.match(regex);
          return isValidPhone ? true : false;
      }

     function validateEmailFromObject(email, callback){
        var result = false;
        if(email){
            if(!_.isEmpty(email.val())){
                var emailAddress = email.val().trim();
                result = validateEmail(emailAddress);
            }
            else{
                result = false;
            }
        }
        else result = true;

        if(result == true){
            $(email).siblings('.incorrectEmail').hide();
            $(email).removeClass('error_input');
        }
        else{
            $(email).siblings('.incorrectEmail').show();
            $(email).addClass('error_input');
        }
        if(callback){
           callback(result);
        }
        return result;
     }
     
     function validatePhoneFromObject(phone, callback) {
         var result = false;
         if (phone){
             if(!_.isEmpty(phone.val())){
                 var phoneNumber = phone.val().trim();
                 result = validatePhone(phoneNumber);
             }
             else{
                 result = false;
             }
         }
         else result = true;

         if(result==false){
            if($(phone).closest('.form-group')){
                $(phone).closest('.form-group').addClass('input-error');
            }else if($(phone).closest('.select-wrap')){
                $(phone).closest('.select-wrap').addClass('input-error');
            }
         }
         else{
            if($(phone).closest('.form-group')){
                $(phone).closest('.form-group').removeClass('input-error');
            }else if($(phone).closest('.select-wrap')){
                $(phone).closest('.select-wrap').removeClass('input-error');
            }
         }
         if(callback){
             callback(result);
         }
         return result;
     }
     function convertStringToDate(dateToConvert){
        var dateFormat = CCRZ.pagevars.pageLabels['Date_Format'],
            dateSplit = dateToConvert.split('/');
        switch(dateFormat){
            case 'dd/mm/yyyy':
                var convertedDate = dateSplit[1] + '/' + dateSplit[0] + '/' + dateSplit[2];
                return new Date(convertedDate);
            case 'yyyy/mm/dd':
                var convertedDate =  dateSplit[1] + '/' + dateSplit[2] + '/' + dateSplit[0];
                return new Date(convertedDate);
            default:
                return new Date(dateToConvert);
        }
     }

    function validateDatepicker(date) {
        var isValidDate = false,
            dateFormat = CCRZ.pagevars.pageLabels['Date_Format'],
            regex = getRegexForFormatDate(dateFormat),
            isValidDate = date.match(regex);
        if (isValidDate){
            var today = new Date(),
                requestedDate = convertStringToDate(date);
            return today < requestedDate;
        }
        return false;
    }
    function getRegexForFormatDate(dateFormat){
         switch(dateFormat){
             //date format: yyyy/mm/dd
             case 'yyyy/mm/dd':
                return /^(?:\d{4}\/(?:(?:(?:(?:0[13578]|1[02])\/(?:0[1-9]|[1-2][0-9]|3[01]))|(?:(?:0[469]|11)\/(?:0[1-9]|[1-2][0-9]|30))|(?:02\/(?:0[1-9]|1[0-9]|2[0-8]))))|(?:(?:\d{2}(?:0[48]|[2468][048]|[13579][26]))|(?:(?:[02468][048])|[13579][26])00)\/02\/29)$/;
             //date format: dd/mm/yyyy
             case 'dd/mm/yyyy':
                return /^(?:(?:(?:0[1-9]|1\d|2[0-8])\/(?:0[1-9]|1[0-2])|(?:29|30)\/(?:0[13-9]|1[0-2])|31\/(?:0[13578]|1[02]))\/[1-9]\d{3}|29\/02\/(?:[1-9]\d(?:0[48]|[2468][048]|[13579][26])|(?:[2468][048]|[13579][26])00))$/;
             //date format: mm/dd/yyyy
             default:
                return /^(((0?[1-9]|1[012])\/(0?[1-9]|1\d|2[0-8])|(0?[13456789]|1[012])\/(29|30)|(0?[13578]|1[02])\/31)\/(19|[2-9]\d)\d{2}|0?2\/29\/((19|[2-9]\d)(0[48]|[2468][048]|[13579][26])|(([2468][048]|[3579][26])00)))$/;
        }
     }

    function removeReadonly(e){
        $(e.target).removeAttr('readonly');
    }

     function setCookie(name, value, days) {
         var d = new Date;
         d.setTime(d.getTime() + 24*60*60*1000*days);
         document.cookie = name + "=" + value + ";path=/;expires=" + d.toGMTString();
     }

     function getCookie(name) {
         var v = document.cookie.match('(^|;) ?' + name + '=([^;]*)(;|$)');
         return v ? v[2] : null;
     }

     function runHandleBarsUtils(){
        Handlebars.registerHelper('ifCond', function (v1, operator, v2, options) {

            switch (operator) {
                case '==':
                    return (v1 == v2) ? options.fn(this) : options.inverse(this);
                case '===':
                    return (v1 === v2) ? options.fn(this) : options.inverse(this);
                case '!=':
                    return (v1 != v2) ? options.fn(this) : options.inverse(this);
                case '!==':
                    return (v1 !== v2) ? options.fn(this) : options.inverse(this);
                case '<':
                    return (v1 < v2) ? options.fn(this) : options.inverse(this);
                case '<=':
                    return (v1 <= v2) ? options.fn(this) : options.inverse(this);
                case '>':
                    return (v1 > v2) ? options.fn(this) : options.inverse(this);
                case '>=':
                    return (v1 >= v2) ? options.fn(this) : options.inverse(this);
                case '&&':
                    return (v1 && v2) ? options.fn(this) : options.inverse(this);
                case '||':
                    return (v1 || v2) ? options.fn(this) : options.inverse(this);
                default:
                    return options.inverse(this);
            }
        });
     }

     function renderCommonHeaderAndFooter(cb){

        if(CCRZ.headerView){
            CCRZ.headerView.model.set("egressThemeUrl", CCRZ.staticResourceUrls.egressTheme);
            var json = (CCRZ.currentUser) ? CCRZ.currentUser : null;
            CCRZ.headerView.model.set("userInfo", json);   
            CCRZ.headerView.prepAndRender();
            setTimeout(() => {
                CCRZ.menuView.render();
                //Locale Switcher
                CCRZ.LocaleSwitcherModel.set("egressThemeUrl", CCRZ.staticResourceUrls.egressTheme);
                CCRZ.localeSwitcherView.switchLocale = function(event){
                    var objLink = $(event.target);
                    var id = objLink.data("id");
                    var i=0;
                    while(i<3 && (!id || id.trim().length <= 0)){
                        objLink = objLink.parent();
                        id = objLink.data("id");
                    }
                    var tbl = $('<tr></tr>').attr({ "class": this.highlightClass, "data-id": id });
                    tbl.appendTo(objLink);
                    var currPageName = CCRZ.pagevars.currentPageName;
                    CCRZ.pagevars.currentPageName = CCRZ.pagevars.currSiteURL + '';
                    this.setLocale(event);
                    setTimeout(() => {
                        CCRZ.pagevars.currentPageName = currPageName;
                    }, 4000);
                }
                CCRZ.localeSwitcherView.render();
                if(cb){
                    cb();
                }
            }, 1000);
        }
        
     }


     
    function showLoader(){
        var loadingParent = $(".main-loader");
        if(loadingParent){
            var loaderClassName = loadingParent.attr("class");
            if(loaderClassName && loaderClassName.indexOf("loading") < 0){
                loadingParent.addClass("loading");
            }
        }
    };
    function hideLoader(){
        var loadingParent = $(".main-loader");
        if(loadingParent){
            var loaderClassName = loadingParent.attr("class");
            if(loaderClassName && loaderClassName.indexOf("loading") >= 0){
                loadingParent.removeClass("loading");
            }
        }
    };

    // Scrolling Util
    // left: 37, up: 38, right: 39, down: 40,
    // spacebar: 32, pageup: 33, pagedown: 34, end: 35, home: 36
    var keys = {37: 1, 38: 1, 39: 1, 40: 1};

    function preventDefault(e) {
    e.preventDefault();
    }

    function preventDefaultForScrollKeys(e) {
    if (keys[e.keyCode]) {
        preventDefault(e);
        return false;
    }
    }

    // modern Chrome requires { passive: false } when adding event
    var supportsPassive = false;
    try {
    window.addEventListener("test", null, Object.defineProperty({}, 'passive', {
        get: function () { supportsPassive = true; } 
    }));
    } catch(e) {}

    var wheelOpt = supportsPassive ? { passive: false } : false;
    var wheelEvent = 'onwheel' in document.createElement('div') ? 'wheel' : 'mousewheel';

    // call this to Disable
    function disableScroll() {
        window.addEventListener('DOMMouseScroll', preventDefault, false); // older FF
        window.addEventListener(wheelEvent, preventDefault, wheelOpt); // modern desktop
        window.addEventListener('touchmove', preventDefault, wheelOpt); // mobile
        window.addEventListener('keydown', preventDefaultForScrollKeys, false);
    }

    // call this to Enable
    function enableScroll() {
        window.removeEventListener('DOMMouseScroll', preventDefault, false);
        window.removeEventListener(wheelEvent, preventDefault, wheelOpt); 
        window.removeEventListener('touchmove', preventDefault, wheelOpt);
        window.removeEventListener('keydown', preventDefaultForScrollKeys, false);
    }

});